/*
 *  sys/hiddi.h
 *
 *  Copyright by QNX Software Systems Limited 1995-2001. All rights reserved.
 */

#include <inttypes.h>
#include <sys/types.h>
#include <sys/queue.h>

#ifndef __HIDDI_SYS_H_INCLUDED
#define __HIDDI_SYS_H_INCLUDED

//#pragma pack(1)
#define HIDD_VERSION				0x0100
#define HID_VERSION					0x0100
#define HIDD_CONNECT_WAIT			60
#define HIDD_CONNECT_WILDCARD		(-1)

#define HIDD_TIME_DEFAULT			0x00000000
#define HIDD_TIME_INFINITY			0xFFFFFFFF

/* flags for report attach		*/
#define HIDD_REPORT_EXCLUSIVE			0x02
#define HIDD_REPORT_FILTER				0x01

/* flags for  report callback	*/
#define HIDD_REPORT_FIRST				0x01
#define HIDD_REPORTS_RESUMED			0x02
#define HIDD_REPORT_BUFFER_OVERFLOW		0x04

#ifdef __QNXNTO__
__BEGIN_DECLS
#else
#ifdef __cplusplus
extern "C" {
#endif
#endif


/* Opaque structures			*/
struct hidd_connection;
struct hidd_device_instance;
struct hidd_report_instance;
struct hidd_report;
struct hidd_report_info;
struct hidd_collection;
struct hidd_field;

typedef struct hidd_button {
	_uint16		usage_page;
	_uint16		usage;

} hidd_button_t;

typedef struct hidd_device_ident {
	_uint32					vendor_id;
	_uint32					product_id;
	_uint32					version;
} hidd_device_ident_t;

typedef struct hidd_device_instance {
	TAILQ_ENTRY(hidd_device_instance)   link;
	_uint32								devno;
	hidd_device_ident_t					device_ident;
	struct hidd_report_info				*report_info;	//preparsed report descriptor data
	_uint16								report_desc_len;
	_uint8								reserved[2];
} hidd_device_instance_t;

typedef struct hidd_funcs {
	_uint32			nentries;
	void			(*insertion)(struct hidd_connection *, hidd_device_instance_t *instance);
	void			(*removal)(struct hidd_connection *, hidd_device_instance_t *instance);
	void			(*report)(struct hidd_connection *, struct hidd_report *handle, void *report_data, _uint32 report_len, _uint32 flags, void *user);
	void			(*event)(struct hidd_connection *, hidd_device_instance_t *instance, _uint16 type);
} hidd_funcs_t;

#define _HIDDI_NFUNCS ((sizeof(hidd_funcs_t) - sizeof(_uint32)) / sizeof(void *))

typedef struct hidd_server_info {
	_uint16					vhid;
	_uint16					vhidd;
} hidd_server_info_t;

typedef struct hidd_connect_parm {
	const char					*path;
	_uint16						vhid;
	_uint16						vhidd;
	_uint32						flags;
	_uint32						evtbufsz;
	hidd_device_ident_t			*device_ident;
	hidd_funcs_t				*funcs;
	_uint16						connect_wait;
	_uint16						_reserved[5];
} hidd_connect_parm_t;


typedef struct hidd_report_props {
	_uint16				usage_page;	
	_uint8				report_id;	
	_uint8				reserved[1];
	_uint16				report_size;
	_uint16				report_count;
	_uint32				data_properties;
/*	hidd_collection_t 	*collection; 		*/
	_uint16				unit_exponent;
	_uint16				unit;
	_int16				logical_min, 		logical_max;
	_int16				physical_min, 		physical_max;
	_uint16				usage_min, 			usage_max;
	_uint16				string_min, 		string_max;
	_uint16				designator_min, 	designator_max;
	_uint8				reserved2[2];
} hidd_report_props_t;

/* defines for data properties                      If bit not set then means */
                                                       
#define HIDD_DATA_PROP_CONSTANT				0x00000001  /* DATA                */
#define HIDD_DATA_VARIABLE					0x00000002  /* ARRAY               */
#define HIDD_DATA_RELATIVE					0x00000004  /* ABSOLUTE            */
#define HIDD_DATA_WRAP						0x00000008  /* NO_WRAP             */
#define HIDD_DATA_NONLINEAR					0x00000010  /* LINEAR              */
#define HIDD_DATA_NO_PREFERRED_STATE		0x00000020  /* PREFERRED_STATE     */
#define HIDD_DATA_NULL_STATE				0x00000040  /* NO NULL STATE       */
#define HIDD_DATA_VOLITILE					0x00000080  /* NONVOLITILE         */
#define HIDD_DATA_BUFFER_BYTES				0x00000100  /* BIT_FIELD           */


/* macros to help interpret data properties flags */
#define HIDD_IS_CONSTANT			( flags )		(flags & HIDD_DATA_CONSTANT) 			? 1 : 0
#define HIDD_IS_DATA				( flags ) 		(flags & HIDD_DATA_CONSTANT) 			? 0 : 1
#define HIDD_IS_VARIABLE			( flags )		(flags & HIDD_DATA_VARIABLE) 			? 1 : 0
#define HIDD_IS_ARRAY				( flags )		(flags & HIDD_DATA_VARIABLE) 			? 0 : 1
#define HIDD_IS_RELATIVE			( flags )		(flags & HIDD_DATA_RELATIVE) 			? 1 : 0
#define HIDD_IS_ABSOLUTE			( flags )		(flags & HIDD_DATA_RELATIVE) 			? 0 : 1
#define HIDD_HAS_WRAP				( flags )		(flags & HIDD_DATA_WRAP) 	 			? 1 : 0
#define HIDD_HAS_NO_WRAP			( flags )		(flags & HIDD_DATA_WRAP)     			? 0 : 1
#define HIDD_IS_NON_LINEAR			( flags )		(flags & HIDD_DATA_NONLINEAR) 			? 1 : 0
#define HIDD_IS_LINEAR				( flags )		(flags & HIDD_DATA_NONLINEAR) 			? 0 : 1
#define HIDD_HAS_NO_PREFERRED_STATE	( flags )		(flags & HIDD_DATA_NO_PREFERRED_STATE) 	? 1 : 0
#define HIDD_HAS_PREFERRED_STATE	( flags )		(flags & HIDD_DATA_NO_PREFERRED_STATE) 	? 0 : 1
#define HIDD_IS_VOLITILE			( flags )		(flags & HIDD_DATA_VOLITILE) 			? 1 : 0
#define HIDD_IS_NONVOLITILE			( flags )		(flags & HIDD_DATA_VOLITILE) 			? 0 : 1
#define HIDD_IS_BUFFERED_BYTES		( flags )		(flags & HIDD_DATA_BUFFERED_BYTES) 		? 1 : 0
#define HIDD_IS_BIT_FIELD			( flags )		(flags & HIDD_DATA_BUFFERED_BYTES) 		? 0 : 1



/* connection functions */
extern int hidd_connect( hidd_connect_parm_t *parm, struct hidd_connection **connection);
extern int hidd_disconnect( struct hidd_connection *connection );

/* report locating */
extern int hidd_get_collections( struct hidd_device_instance *report_info, struct hidd_collection *search, struct hidd_collection ***found, _uint16 *num);
extern int hidd_collection_usage( struct hidd_collection *collection, _uint16 *usage_page, _uint16 *usage);
extern int hidd_get_report_instance( struct hidd_collection *collection, _uint16 report_index, _uint16 report_type, struct hidd_report_instance **report_inst );
extern int hidd_report_collection( struct hidd_report_instance *report, struct hidd_collection **collection );
extern int hidd_get_num_props( struct hidd_report_instance *report_inst, _uint16 *num );
extern int hidd_get_report_props( struct hidd_report_instance *report_inst, hidd_report_props_t *report_props, _uint16 *len);

/* report registration */
extern int hidd_report_attach( struct hidd_connection *connection, struct hidd_device_instance *device,
	struct hidd_report_instance *report_inst, _uint32 flags, size_t extra, struct hidd_report **report );
extern int hidd_report_detach( struct hidd_report *report ); 
extern int hidd_reports_detach( struct hidd_connection *connection, struct hidd_device_instance *device_instance);
extern void *hidd_report_extra(struct hidd_report *report);

extern int hidd_get_idle( struct hidd_report *handle, _uint16 *idle_rate);
extern int hidd_set_idle( struct hidd_report *handle, _uint16 idle_rate);

/* report retrieval/sending */
extern int hidd_get_report( struct hidd_report *report , _uint8 *report_data );
extern int hidd_send_report( struct hidd_report *handle , void *report_data );

/* report buffer allocation */
extern int hidd_report_len( struct hidd_report_instance *report_inst, _uint16 *report_len);
extern int hidd_num_buttons( struct hidd_report_instance *report_inst, _uint16 *max_buttons);
extern int hidd_num_values( struct hidd_report_instance *report_inst, _uint16 *max_values);

/*  data extraction/insertion for reports */
extern int hidd_get_buttons( struct hidd_report_instance *report , struct hidd_collection *col, _uint16 usage_page , void *report_data , _uint16 *usages , _uint16 *len );
extern int hidd_get_all_buttons( struct hidd_report_instance *report , void *report_data , hidd_button_t *list , _uint16 *len );
extern int hidd_button_list_diff(_uint16 *list1 , _uint16 *list2 , _uint16 *diff , _uint16 *len );
extern int hidd_button_list_same(_uint16 *list1 , _uint16 *list2 , _uint16 *same , _uint16 *len );
extern int hidd_get_usage_value( struct hidd_report_instance *report , struct hidd_collection *col , _uint16 usage_page , _uint16 usage , void *report_data , _uint32 *usage_value);
extern int hidd_get_scaled_usage_value( struct hidd_report_instance *report , struct hidd_collection *col , _uint16 usage_page , _uint16 usage , void *report_data , _int32 *usage_value );
extern int hidd_set_buttons( struct hidd_report_instance *report ,struct hidd_collection *col, _uint16 usage_page , _uint16 *usage_list , _uint16 len , _uint8 *report_data , _uint16 buff_len );
extern int hidd_set_usage_value( struct hidd_report_instance *report, struct hidd_collection *col, _uint16 usage_page, _uint16 usage, _uint32 usage_value, _uint8 *report_data, _uint16 len );

extern int hidd_server_info( struct hidd_connection *connnection, hidd_server_info_t *info );

/* set device protocol  */
extern int hidd_get_protocol( struct hidd_connection *connnection, struct hidd_device_instance * dev_instance, _uint8 *protocol_id);
extern int hidd_set_protocol( struct hidd_connection *connnection, struct hidd_device_instance * dev_instance, _uint8 protocol_id);

/* device string functions */
extern int hidd_get_manufacturer_string( struct hidd_connection *connnection, struct hidd_device_instance *dinst, void *buffer, _uint16 blen );
extern int hidd_get_product_string( struct hidd_connection *connnection, struct hidd_device_instance *dinst, void *buffer, _uint16 blen );
extern int hidd_get_serial_number_string( struct hidd_connection *connnection, struct hidd_device_instance *dinst, void *buffer, _uint16 blen );
extern int hidd_get_indexed_string( struct hidd_connection *connnection, struct hidd_device_instance *dinst, _uint16 index, void *buffer, _uint16 blen );

#ifdef __QNXNTO__
__END_DECLS
#else
#ifdef __cplusplus
};
#endif
#endif

//#pragma pack()

#endif
