/*
 *  sys/io-hid.h
 *
 *  Copyright by QNX Software Systems Limited 1995-2000. All rights reserved.
 */

#ifndef __NTO_IO_HID_H_INCLUDED
#define __NTO_IO_HID_H_INCLUDED

#include <sys/types.h>
#include <sys/queue.h>
#include <sys/dispatch.h>
#include <sys/iofunc.h>
#include <inttypes.h>
#include <sys/hiddi.h>

__BEGIN_DECLS


struct hiddi_device_instance;

// defines for string callout
#define HIDD_STRING_MANUFACTURER 		0x01
#define HIDD_STRING_PRODUCT		 		0x02
#define HIDD_STRING_SERIAL_NUMBER 		0x03

#define HIDD_VENDOR_UNKNOWN				0xFFFF
#define HIDD_PRODUCT_UNKNOWN			0xFFFF
#define HID_VERSION_UNKNOWN				0xFFFF

/*
 * The following are entry points a module provides into itself
 * when it registers.  Will be called by io-hid.
 */

typedef struct _io_hid_registrant_funcs {
	_uint32		nfuncs;
	int     	(*client_attach)	(int reg_hdl, void *user);
	int     	(*client_detach)	(int reg_hdl, void *user);

	int			(*rbuffer_alloc)	(_uint16 size, void **bptr);
	int			(*rbuffer_free)		(void *bptr);

	int     	(*report_read)  	(int reg_hdl, void *user, _uint8 rid, _uint16 rtype, void *data, _uint16 dlen);
	int     	(*report_write)  	(int reg_hdl, void *user, _uint8 rid, _uint16 rtype, void *data, _uint16 dlen);

	int     	(*get_idle)			(int reg_hdl, void *user, _uint8 rid, _uint16 *idle_rate);
	int     	(*set_idle)			(int reg_hdl, void *user, _uint8 rid, _uint16 idle_rate);

	int     	(*get_protocol)		(int reg_hdl, void *user, _uint8 *protocol);
	int     	(*set_protocol)		(int reg_hdl, void *user, _uint8 protocol);

	int			(*string)			(int reg_hdl, void *user, _uint16 request, void **str);
	int			(*indexed_string)	(int reg_hdl, void *user, _uint16 index, void **str);

	int     	(*reset)			(int reg_hdl, void *user);
	int			(*reserved)			(void *ptr);
} io_hid_registrant_funcs_t;

#define _IO_HID_REG_NFUNCS		((sizeof(io_hid_registrant_funcs_t)-sizeof(u_int))/sizeof(void *))


typedef struct _io_hid_registrant {
	uint32_t                  		flags; 
	hidd_device_ident_t 			*device_ident;
	void							*desc;		// describes device and data it sends/receives
	_uint16							dlen;
	_uint8							reserved[2];
	void                      		*user_hdl;
	io_hid_registrant_funcs_t 		*funcs;
	_uint8                     		reserved2[4];
} io_hid_registrant_t;


/*
 * List of calls drivers can make into io-hid
 */

typedef struct _io_hid_self {
	_uint32		nfuncs;
	int			(*reg)     		(void *dll_hdl, io_hid_registrant_t *registrant, int *reg_hdlp );
	int			(*dereg)		(int registrant_hdl);
	int			(*get_buffer)	( int reg_hdlp, void **buffer );
	int			(*send_report)	(int registrant_hdl, void *data, _uint32 dlen );
} io_hid_self_t;

#define _IO_HID_NFUNCS		((sizeof(io_hid_self_t)-sizeof(u_int))/sizeof(void *))


/*
 * Entry point for DLLs
 */

typedef struct _io_hid_dll_entry {
	char	*name;
	int 	nfuncs;
	int 	(*init) (void *dll_hdl, dispatch_t *dpp, io_hid_self_t *ioh, char *options);
	int 	(*shutdown) (void *dll_hdl);
} io_hid_dll_entry_t;

__END_DECLS

#endif
