/*
 * Copyright 2001, QNX Software Systems Ltd. All Rights Reserved
 *
 * This source code has been published by QNX Software Systems Ltd. (QSSL).
 * However, any use, reproduction, modification, distribution or transfer of
 * this software, or any software which includes or is based upon any of this
 * code, is only permitted under the terms of the QNX Open Community License
 * version 1.0 (see licensing.qnx.com for details) or as otherwise expressly
 * authorized by a written license agreement from QSSL. For more information,
 * please email licensing@qnx.com.
 */

#ifndef __KEYMAP_H_INCLUDED__
#define __KEYMAP_H_INCLUDED__

#ifndef _USBCODES_H
#include <sys/usbcodes.h>
#endif

#define KEY_PRESS     (1)
#define KEY_RELEASE   (0)

#define IGNORE_COMP     0x01
#define IGNORE_ACTIONS  0x02

#define DONT_NEED_CAP   0
#define NEED_CAP        1

// Keyboard switchers
#define LEFT_ALT_SHIFT          (1)
#define RIGHT_ALT_SHIFT         (2)
#define LEFT_CTRL_SHIFT         (3)
#define RIGHT_CTRL_SHIFT        (4)

#ifndef _STORAGE_H

//struct _modArray;
typedef struct _modArray * pmod_array_t; 
//struct _actArray;
typedef struct _actArray *  pact_array_t; 
//struct _compArea;
typedef struct _compArea *  pcomp_area_t; 
//struct _keyMapDir;
typedef struct _keyMapDir *  pkey_map_dir_t; 

/* structure defines keyboard layout description file header                        */
typedef struct _stHeader
{
   _uint8 leds[8];                /* table to get LEDs                              */
   _uint8 stdModArray[32];        /* table to re-map to standard modifiers          */
   pmod_array_t pMod;             /* virtual modifiers                              */
   pact_array_t pAct;             /* actions that change modifier state             */
   pkey_map_dir_t pKMapDir;       /* key map directory                              */
   pcomp_area_t  pComp;           /* area where combinations of codes are located   */
                                  /* this area is used for mapping of composed      */
                                  /* characters sequences                           */
} st_header_t, * pst_header_t;
#endif

typedef struct _kmap_data {
        struct _stHeader  header;
        USBKCode          lastKey; 
        char              *name;
} kmap_data, * p_kmap_data;


/* Using USB code and current modifiers state, function resolves  UNICODE string    */
void kmap_get_unicode(pst_header_t pHeader,USBKCode code, int bNeedCAP, wchar_t * pCodes, int * pnLen, int * pIsDeadKey);

/* function processes keyboard activity                                             */
void kmap_process_key_event(pst_header_t pHeader, USBKCode code, int nMakeBreak, 
			    wchar_t * pBuf, int * pnBufLen);

/* function checks if this is a deadkey code                                        */
int  kmap_is_dead_key(pst_header_t pHeader, wchar_t cDeadKeyCode);

/* function returns bit array of standard modifiers                                 */
_uint32 kmap_get_std_modifiers(pst_header_t pHeader);

/* function returns bit array of LEDs                                               */
_uint8 kmap_get_leds(pst_header_t pHeader);

/* function sets modifiers' state using new LED state                               */
void kmap_set_leds(pst_header_t pHeader, _uint8 leds);

/* function copies keyboard state                                                   */
void kmap_copy_keyboard_state(pst_header_t pFrom, pst_header_t pTo);

/* function reset keyboard state                                                    */
void kmap_reset_keyboard_state(pst_header_t  pHeader);

/* function loads new keyboard definition file                                      */
int kmap_load_kbd_file(const char * fname, pst_header_t pHeader);

/* free allocated memory we don't need more                                         */
int kmap_free_memory(pst_header_t pHeader);

/* load keyboard mapping files                                                      */
int kmap_load_keyboards(p_kmap_data * ppData, char * proc_path);

/* Load new mapping file                                                            */
int kmap_add_new_keyboard( char * name, p_kmap_data * ppData);

/* Remove keyboard mapping file from memory                                         */
int kmap_remove_keyboard(char * name);

/* Make another mapping file active                                                 */
int kmap_switch_to_keyboard(char * name, p_kmap_data * ppData);

/* Switch to next keyboard                                                          */
void kmap_switch_to_next(p_kmap_data * ppData);

/* Number of loaded keyboards                                                       */
int kmap_get_num_of_loaded_keyboards();

/* Get keymap descriptor                                                            */
int kmap_get_kmap_data_by_name(char * name, p_kmap_data * ppData);

/* Get keymap descriptor                                                            */
int kmap_get_kmap_data_by_number(int nKbd, p_kmap_data * ppData);

/* Get current switcher (see constants in the beginning of this file                */
int kmap_get_active_switcher();

#endif /* __KEYMAP_H_INCLUDED                                                       */
