/*

Copyright 2000, QNX Software Systems Ltd. Unpublished Work All Rights
Reserved.

 
This source code contains confidential information of QNX Software Systems
Ltd. (QSSL). Any use, reproduction, modification, disclosure, distribution
or transfer of this software, or any software which includes or is based
upon any of this code, is only permitted under the terms of the QNX
Confidential Source License version 1.0 (see licensing.qnx.com for details)
or as otherwise expressly authorized by a written license agreement from
QSSL. For more information, please email licensing@qnx.com.

*/
#ifndef _VR4122_H_INCLUDED
#define _VR4122_H_INCLUDED

#ifndef __PLATFORM_H_INCLUDED
#include <sys/platform.h>
#endif

#ifndef _INTTYPES_H_INCLUDED
 #include <inttypes.h>
#endif

#ifndef __TYPES_H_INCLUDED
 #include <sys/types.h>
#endif

#include <hw/pci.h>
#include <hw/pci_devices.h>
#include <pcidrvr.h>

typedef struct _vr4122_dev {
	uint32_t		vr4122_base;
	char			*cnfg;
	char			*base;
	unsigned		flags;
	uint64_t		pci_cnfg;
	uint64_t		pci_isa;
	uint64_t		pci_io;
	uint64_t		pci_mem;
	uint8_t			revision;
	uint8_t			rsvd[3];
} vr4122_dev_t;


#define EAGLE_ONBOARD_BASE			0x0dffffa0
#define EAGLE_ONBOARD_LEN			0x50

#define 	VR4122_CNFG_BASE				0x0f000c00
#define 	VR4122_CNFG_LEN				0x000001ff 

#define VR4122_PCIMEM_BASE			0x10000000 		
#define VR4122_PCIMEM_SIZE			0x05ffffff

#define VR4122_PCIIO_BASE			0x16000000
#define VR4122_PCIIO_SIZE			0x01ffffff

/*
#define VR4122W(x)              		*(volatile uint32_t *)(pdev->cnfg + (x))
#define VR4122_PCIMMAW1REG			VR4122W(0x00)
#define VR4122_PCIMMAW2REG			VR4122W(0x04)
#define VR4122_PCITAW1REG			VR4122W(0x08)
#define VR4122_PCITAW2REG			VR4122W(0x0c)
#define VR4122_PCIMIOAWREG			VR4122W(0x10)
#define VR4122_PCICONFDREG			VR4122W(0x14)
#define VR4122_PCICONFAREG			VR4122W(0x18)
#define VR4122_PCIMAILREG			VR4122W(0x1c)
#define VR4122_BUSERRADREG			VR4122W(0x24)
#define VR4122_INTCNTSTAREG			VR4122W(0x28)
#define VR4122_PCIEXACCREG			VR4122W(0x2c)
#define VR4122_PCIRECONTREG			VR4122W(0x30)
#define VR4122_PCIENREG  			VR4122W(0x34)
#define VR4122_PCICLKSELREG			VR4122W(0x38)
#define VR4122_PCITRDYVREG			VR4122W(0x3c)
#define VR4122_PCICLKRUNREG			VR4122W(0x60)
#define VR4122_VENDORIDREG			VR4122W(0x100)
#define VR4122_COMMANDREG			VR4122W(0x104)
#define VR4122_REVIDREG				VR4122W(0x108)
#define VR4122_CACHELSREG       		VR4122W(0x10c)
#define VR4122_MAILBAREG				VR4122W(0x110)
#define VR4122_PCIMBA1REG			VR4122W(0x114)
#define VR4122_PCIMBA2REG			VR4122W(0x118)
#define VR4122_INTLINEREG			VR4122W(0x13c)
#define VR4122_RETVALREG				VR4122W(0x140)

#define EAGLE_ONBOARDH(x)              *(volatile uint16_t *)(pdev->base + (x))
#define EAGLE_ONBOARDB(x)              *(volatile uint8_t *)(pdev->base + (x))
#define EAGLE_ONBOARD_FLSHCNT          EAGLE_ONBOARDB(0x00)
#define EAGLE_ONBOARD_FLSHBANK         EAGLE_ONBOARDB(0x04)
#define EAGLE_ONBOARD_SWTCHSET         EAGLE_ONBOARDB(0x08)
#define EAGLE_ONBOARD_LEDWR1           EAGLE_ONBOARDB(0x20)
#define EAGLE_ONBOARD_LEDWR2           EAGLE_ONBOARDB(0x24)
#define EAGLE_ONBOARD_SDBINT           EAGLE_ONBOARDB(0x30)
#define EAGLE_ONBOARD_SDBINTMSK        EAGLE_ONBOARDB(0x34)
#define EAGLE_ONBOARD_RSTREG           EAGLE_ONBOARDB(0x38)
#define EAGLE_ONBOARD_PCIINTREG        EAGLE_ONBOARDB(0x3c)
#define EAGLE_ONBOARD_PCIINTMASKREG    EAGLE_ONBOARDB(0x40)
#define EAGLE_ONBOARD_CLKDIV           EAGLE_ONBOARDB(0x44)
#define EAGLE_ONBOARD_REVSION          EAGLE_ONBOARDB(0x48)
*/

int	 vr4122_attach( char *options, void **hdl );
int	 vr4122_detach( void *hdl );
int	 vr4122_cnfg_bridge( void *hdl, uint32_t bus, uint32_t devfunc, pci_bus_t *pbus );
int	 vr4122_read_cnfg( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t reg, uint32_t width, void *buf );
int	 vr4122_write_cnfg( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t reg, uint32_t width, const uint32_t data );
int	 vr4122_special_cycle( void *hdl, uint32_t bus, uint32_t data );
int vr4122_map_addr( void *hdl, uint64_t iaddr, uint64_t *oaddr, uint32_t type );
int vr4122_map_irq( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t line, uint32_t pin, uint32_t flags );
int vr4122_avail_irq( void *hdl, uint32_t bus, uint32_t devfunc, uint32_t *list, uint32_t *nelm );

#endif
