/*
   Copyright 2001, QNX Software Systems Ltd. All Rights Reserved

   This source code has been published by QNX Software Systems Ltd. (QSSL).
   However, any use, reproduction, modification, distribution or transfer of
   this software, or any software which includes or is based upon any of this
   code, is only permitted under the terms of the QNX Community License version
   1.0 (see licensing.qnx.com for details) or as otherwise expressly authorized
   by a written license agreement from QSSL. For more information, please email
   licensing@qnx.com.
   */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"
#include "globals.h"


int CreateCoordinateRegion( PtWidget_t *widget, ApInfo_t *apinfo, 
                            PtCallbackInfo_t *cbinfo )
{
    PtArg_t         argt[8];
    PhArea_t        area = { {0,0},{SHRT_MAX, SHRT_MAX} };
    PhRegion_t      region;
    PhRid_t         rid;
    PhWindowEvent_t ev;
    PhArea_t        *old_area;
    PhArea_t        new_area;
    int             width, height;

    ev.event_f = Ph_WM_CONSWITCH;
    ev.rid = PtWidgetRid(widget);
    ev.input_group = 1;
    ev.event_state = 0;
    ev.pos.x = 0;
    ev.pos.y = 0;
    PtForwardWindowEvent( &ev );
    
    // find the graphics region
    PhQueryRids(Ph_RIDQUERY_TOWARD, 0, 0, Ph_GRAFX_REGION, 0, 0, 0, &rid, 1);

    /* set up the main region */
//  memcpy(&area, &Screen, sizeof(PhRect_t));
    PtSetArg( &argt[0], Pt_ARG_AREA, &area, 0 );
    PtSetArg( &argt[1], Pt_ARG_REGION_SENSE, -1, 
              Ph_EV_BUT_PRESS | Ph_EV_BUT_RELEASE );
    PtSetArg( &argt[2], Pt_ARG_REGION_OPAQUE, -1, 
              Ph_EV_PTR_ALL | Ph_EV_EXPOSE | Ph_EV_DRAW);
    PtSetArg( &argt[3], Pt_ARG_FILL_COLOR, 0xd3d4cb, 0 );
    PtSetArg( &argt[4], Pt_ARG_REGION_FIELDS, -1, Ph_REGION_ORIGIN 
            | Ph_REGION_RECT | Ph_REGION_EV_SENSE | Ph_REGION_EV_OPAQUE 
            | Ph_REGION_PARENT | Ph_REGION_FLAGS );
    PtSetArg( &argt[5], Pt_ARG_REGION_FLAGS, -1, Ph_FORCE_FRONT );
    PtSetArg( &argt[6], Pt_ARG_REGION_PARENT, 0, 0 );
    PtSetParentWidget( NULL );
    TouchRegion = PtCreateWidget( PtRegion, NULL, 7, argt );

    // add the button press handler  
    PtAddEventHandler( TouchRegion, Ph_EV_BUT_RELEASE, Touch, NULL ); 

    // grab the dimensions of the graphics region
    if (Screen.lr.x == 0)
        PhRegionQuery(rid, &region, &Screen, NULL, 0);

    // open a picture database 
    AppDatabase = ApOpenDBase(ABM_acalib_gallery);
    PtSetParentWidget( TouchRegion );

    width = Screen.lr.x - Screen.ul.x + 1;
    height = Screen.lr.y - Screen.ul.y + 1;

    // create all my widgets 
    Directions = ApCreateWidget( AppDatabase, "directions", 0, 0, 0, NULL );
    PtRealizeWidget( Directions );
    PtGetResource( Directions, Pt_ARG_AREA, &old_area, NULL );
    memcpy( &new_area, old_area, sizeof( new_area ) );

    new_area.pos.x = ( width -  new_area.size.w ) / 2;
    new_area.pos.y = ( height - new_area.size.h) / 2;
    PtSetResource( Directions, Pt_ARG_AREA, &new_area, NULL );

    // get the target images
    TargetImages[0] = ApGetImageRes(AppDatabase, "bull1");
    TargetImages[1] = ApGetImageRes(AppDatabase, "bull2");
    TargetImages[2] = ApGetImageRes(AppDatabase, "bull3");

    TargetDim = TargetImages[0]->size.w;

    // save the positions for all the press coordinates
    TargetPos[STATE_UL_PRESS].x = Screen.ul.x + TargetOffset;
    TargetPos[STATE_UL_PRESS].y = Screen.ul.y + TargetOffset;

    TargetPos[STATE_LL_PRESS].x = Screen.ul.x + TargetOffset;
    TargetPos[STATE_LL_PRESS].y = Screen.lr.y - TargetDim - TargetOffset;

    TargetPos[STATE_LR_PRESS].x = Screen.lr.x - TargetDim - TargetOffset;
    TargetPos[STATE_LR_PRESS].y = Screen.lr.y - TargetDim - TargetOffset;

    TargetPos[STATE_UR_PRESS].x = Screen.lr.x - TargetDim - TargetOffset;
    TargetPos[STATE_UR_PRESS].y = Screen.ul.y + TargetOffset;

    // create the target widget and set it up for the first press
    TargetWidget = ApCreateWidget(AppDatabase,"target_label", 0, 0, 0, NULL);
    PtRealizeWidget( TargetWidget );

    CurrentTarget = 0;
    State = STATE_UL_PRESS;
    set_target_position();
  
    return( Pt_CONTINUE );
}

