/*
  Copyright 2001, QNX Software Systems Ltd. All Rights Reserved

   This source code has been published by QNX Software Systems Ltd. (QSSL).
  However, any use, reproduction, modification, distribution or transfer of
  this software, or any software which includes or is based upon any of this
  code, is only permitted under the terms of the QNX Community License version
  1.0 (see licensing.qnx.com for details) or as otherwise expressly authorized
  by a written license agreement from QSSL. For more information, please email
  licensing@qnx.com.
*/

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <stddef.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>
#include <process.h>
#include <math.h>
#include <sys/stat.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>
#include <photon/PhSystem.h>
#include <photon/PhService.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"
#include "globals.h"

#ifndef Ph_EV_SERVICE
#define Ph_EV_SERVICE   0x00040000
#endif


//#define CALIB_DEBUG 1

// set the new position of the target widget and reset the image
void set_target_position( void )
{
    PtArg_t args[2];

    CurrentTarget = 0;
    PtSetArg( &args[0], Pt_ARG_POS, &TargetPos[State], 0 );
    PtSetArg( &args[1], Pt_ARG_LABEL_DATA, TargetImages[CurrentTarget], 
              sizeof(PhImage_t) );
    PtSetResources( TargetWidget, 2, args );
}

// put pointer backintocookedmode
void reset_pointer( void )
{
    PhRect_t rect = {{0, 0}, {SHRT_MAX, SHRT_MAX}};
    PhEvent_t               ev;
    static PhPointerCtrlEvent_t         PtrCtrlEvent;

    memset(&PtrCtrlEvent, 0, sizeof(PtrCtrlEvent));
    PtrCtrlEvent.type = POINTER_CTRL_FILTER;
    PtrCtrlEvent.len = sizeof(PtrCtrlEvent.ctrl.filter);
    PtrCtrlEvent.ctrl.filter.flags = POINTER_CTRL_ON;

    memset(&ev,0,sizeof(ev));
    ev.type = Ph_EV_SERVICE;
    ev.subtype = Ph_EV_POINTER_CTRL;
    ev.flags = Ph_EMIT_TOWARD;
    ev.num_rects = 1;
    ev.data_len = sizeof(PtrCtrlEvent);

    PhEventEmit(&ev, &rect, &PtrCtrlEvent);
}


//
// button press event handler, called when the user hits the target
//
// the person presses in the center of a target so we must move the point
// by half the target size
//
int Touch( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
{
    PtArg_t            args[1];
    PhPointerEvent_t   *EventPointer;
    int                result_calibrate;
    static int         x[4],y[4];
    int                ULx, ULy;
    int                LRx,LRy;
    int                width, height;
    PhArea_t           new_area, *old_area;
    int                i;
    int                swap;
    int                x_offset, y_offset;
    float              screen_offset; 
    float              low,high;
    int                xdist,ydist;

    if (cbinfo->event->subtype != Ph_EV_RELEASE_REAL)
        return (Pt_CONTINUE);

    EventPointer = (PhPointerEvent_t *)PhGetData(cbinfo->event);
    switch(State) 
    {
        case STATE_UL_PRESS:
        case STATE_LL_PRESS:
        case STATE_LR_PRESS: 
            x[State] = EventPointer->pos.x;
            y[State] = EventPointer->pos.y;
            State++;
            set_target_position();

            break;

        case STATE_UR_PRESS:  
            x[State] = EventPointer->pos.x;
            y[State] = EventPointer->pos.y;
            State = STATE_DONE;

            if( Verbose )
            {
                for( i=0; i<4; i++ )
                   printf( "Point %d: %d,%d\n", i, x[i], y[i] );
            }


            PtUnrealizeWidget( TargetWidget );   
            PtTimerArm( ABW_timer, 0 );
            PtTimerArm( ABW_done_timer, 1000 );

            reset_pointer();

            /* There are 4 cases, just handle them all separatly */

            xdist = ( Screen.lr.x - Screen.ul.x + 1 ) - 
                    ( 2 * ( (TargetDim/2) + TargetOffset ) );
            ydist = ( Screen.lr.y - Screen.ul.y + 1 ) - 
                    ( 2 * ( (TargetDim/2) + TargetOffset ) );

            screen_offset =  (float)( (TargetDim/2) + TargetOffset );



            /* average out the x and y values */
            if(abs(x[0] - x[1]) < abs(y[0] - y[1]))
              {
              ULx = (x[0] + x[1]) / 2;
              ULy = (y[0] + y[3]) / 2;

              LRx = (x[2] + x[3]) / 2;
              LRy = (y[1] + y[2]) / 2;
              swap = 0;
              }
            else
              {
              ULx = (y[0] + y[1]) / 2;
              ULy = (x[0] + x[3]) / 2;

              LRx = (y[2] + y[3]) / 2;
              LRy = (x[1] + x[2]) / 2;
              swap = 1;
              }            
            
            
            x_offset = abs((float)( ULx - LRx )) / (float)((0 == swap) ? xdist : ydist ) *
                       screen_offset;
            y_offset = abs((float)( ULy - LRy )) / (float)((0 == swap) ? ydist : xdist ) *
                       screen_offset;         
                       

            if (ULx < LRx)
               {
               ULx -= x_offset;
               LRx += x_offset;
               }
            else
               {
               ULx += x_offset;
               LRx -= x_offset;
               }

            if (ULy < LRy)
               {
               ULy -= y_offset;
               LRy += y_offset;
               }
            else
               {
               ULy += y_offset;
               LRy -= y_offset;
               }
                       

#ifdef CALIB_DEBUG
            printf("Raw: UL (%d, %d) LR (%d, %d)\n", \
                    ULx, ULy, LRx, LRy);
#endif            
            
            result_calibrate = calibrate( &Screen, ULx, ULy, LRx, LRy, swap );

            width = Screen.lr.x - Screen.ul.x + 1;
            height = Screen.lr.y - Screen.ul.y + 1;

            // display the button to exit
            PtSetParentWidget( TouchRegion );
            DoneWidget = ApCreateWidgetFamily( AppDatabase, "done_grp", 
                                               0, 0, 0, NULL );
            PtRealizeWidget( DoneWidget );
            PtGetResource( DoneWidget, Pt_ARG_AREA, &old_area, NULL );
            memcpy( &new_area, old_area, sizeof( new_area ) );

            PtGetResource( Directions, Pt_ARG_AREA, &old_area, NULL );

            new_area.pos.x = old_area->pos.x;
            new_area.pos.y = old_area->pos.y;
            PtSetResource( DoneWidget, Pt_ARG_AREA, &new_area, NULL );

            break;

        case STATE_DONE:
            break;

        default:
            State = STATE_UL_PRESS;
            break;
    }

    return( Pt_CONTINUE );
}


int calibrate( PhRect_t *rect, int xoffset, int yoffset, 
               int xscale, int yscale, int swap)
{
    int        reslx, resly, reshx, reshy;
    PhEvent_t   ev;
    static PhPointerCtrlEvent_t PtrCtrlEvent;
    FILE        *fp;

    reslx = rect->ul.x;
    resly = rect->ul.y;
    reshx = rect->lr.x;
    reshy = rect->lr.y;

    // Re-establish transformation matrix
    memset(&PtrCtrlEvent, 0, sizeof(PtrCtrlEvent));
    PtrCtrlEvent.type = POINTER_CTRL_FILTER;
    PtrCtrlEvent.len = sizeof(PtrCtrlEvent.ctrl);
    PtrCtrlEvent.ctrl.filter.flags = (POINTER_CTRL_SETMATRIX |
                    POINTER_CTRL_DEVMATRIX |
                    POINTER_CTRL_SCRNMATRIX);

    if( swap )
        PtrCtrlEvent.ctrl.filter.flags |= POINTER_CTRL_SWAP;

    PtrCtrlEvent.ctrl.filter.scrn.xoffset.number = reslx;
    PtrCtrlEvent.ctrl.filter.scrn.yoffset.number = resly;
    PtrCtrlEvent.ctrl.filter.scrn.xscale.number = reshx;
    PtrCtrlEvent.ctrl.filter.scrn.yscale.number = reshy;

    PtrCtrlEvent.ctrl.filter.coord.xoffset.number = xoffset;
    PtrCtrlEvent.ctrl.filter.coord.yoffset.number = yoffset;
    PtrCtrlEvent.ctrl.filter.coord.xscale.number = xscale;
    PtrCtrlEvent.ctrl.filter.coord.yscale.number = yscale;

    memset(&ev, 0, sizeof(ev));
    ev.type = Ph_EV_SERVICE;
    ev.subtype = Ph_EV_POINTER_CTRL;
    ev.flags = Ph_EMIT_TOWARD;
    ev.data_len = sizeof(PtrCtrlEvent);

    PhEventEmit(&ev, NULL, &PtrCtrlEvent);

    if( Verbose )
    {
        printf( "Calibration: %dx%d:%dx%d:%d %d %d %d %d\n",
                reslx, resly, reshx, reshy, xoffset, xscale, 
                yoffset, yscale, swap);
    }

    // write the config file
    if ((fp = fopen(CalibFile,"w")) == NULL) 
    {
        return (Pt_CONTINUE);
    }

    fprintf(fp ,"%dx%d:%dx%d:%d %d %d %d %d\n",
        reslx,resly,reshx,reshy,xoffset,xscale,yoffset,yscale,swap);
  
    fclose(fp);

    return (Pt_CONTINUE);
}


int timer_activate( PtWidget_t *widget, ApInfo_t *apinfo, 
                    PtCallbackInfo_t *cbinfo )
{
    PtArg_t args[1];

    CurrentTarget = (CurrentTarget + 1) % 3;
    PtSetArg( &args[0], Pt_ARG_LABEL_DATA, 
              TargetImages[CurrentTarget], sizeof(PhImage_t) );
    PtSetResources( TargetWidget, 1, args );
 
    PtTimerArm( ABW_timer, 500 );
  
    return( Pt_CONTINUE );
}
