/*
   Copyright 2001, QNX Software Systems Ltd. All Rights Reserved

   This source code has been published by QNX Software Systems Ltd. (QSSL).
   However, any use, reproduction, modification, distribution or transfer of
   this software, or any software which includes or is based upon any of this
   code, is only permitted under the terms of the QNX Community License version
   1.0 (see licensing.qnx.com for details) or as otherwise expressly authorized
   by a written license agreement from QSSL. For more information, please email
   licensing@qnx.com.
   */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#ifdef __QNXNTO__
    #include <sys/param.h>
#endif

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"

/* Define the Global Data - only 1 .c file does this */
#define DEFINE_GLOBALS
#include "globals.h"

/* Application Options string */
const char ApOptions[] =
    AB_OPTIONS ":co:d:v"; /* Add your options in the "" */

typedef struct input_defaults_t
{
    int state;
    PhRect_t rect;
    int xoffset, yoffset;
    int xscale, yscale;
    int swap;
} InputDefaults_t;

InputDefaults_t def;


int init( int argc, char *argv[] )
{
    int c;
    int force = 1;
    int w, h;
    FILE *fp = NULL;
    #ifdef __QNXNTO__
    char name[MAXHOSTNAMELEN];
    #endif

    memset(&Screen, 0, sizeof(PhRect_t));
    Verbose = 0;

    def.state=0;
    TargetOffset = 5;

    while ((c = getopt(argc, argv, ApOptions)) != -1)
    {
        switch (c)
        {
            case 'v':
                Verbose++;
                break;

            case 'c': // do not run if a config file exists
                force = 0;
                break;

            case 'o': // offset for crosshair position, tweek for calibration
                TargetOffset = strtol( optarg, 0, 0 );
                break;

            case 'd':
                if (sscanf(optarg, "%d,%d", &w, &h) != 2)
                {
                    printf( "Invalid arguments to -d option.  
                             Check useage message.\n");
                    exit (-1);
                }
                Screen.lr.x = w;
                Screen.lr.y = h;
                break;
        }
    }

#ifdef __QNXNTO__
    if (gethostname(name, MAXHOSTNAMELEN) == -1)
        strcpy(name, "localhost");
    sprintf(CalibFile, NTO_CONFIG_FILE, name);
#else
    sprintf(CalibFile, QNX4_CONFIG_FILE, getnid());
#endif
    
    fp = fopen (CalibFile, "r");
    if (!force && fp)
    {
        fclose (fp);
        exit (0);
    }
    fclose (fp);

    return( Pt_CONTINUE );
}

#ifndef Ph_EV_SERVICE
#define Ph_EV_SERVICE               0x00040000
#endif

// signal handler to free shared memory
void base_signal( int sig_no )
{
    reset_pointer();

    if (State != STATE_DONE)
        exit( EXIT_SUCCESS );
}


void pointer_raw( void )
{
  static PhPointerCtrlEvent_t   PtrCtrlEvent;
  PhEvent_t         ev;

  // throw Pointer into "raw" mode
  memset(&PtrCtrlEvent, 0, sizeof(PhPointerCtrlEvent_t));
  PtrCtrlEvent.type = POINTER_CTRL_FILTER;
  PtrCtrlEvent.len = sizeof(PtrCtrlEvent.ctrl.filter);
  PtrCtrlEvent.ctrl.filter.flags = POINTER_CTRL_OFF;
  
  memset(&ev, 0, sizeof(PhEvent_t));
  ev.type = Ph_EV_SERVICE;
  ev.subtype = Ph_EV_POINTER_CTRL;
  ev.flags = Ph_EMIT_TOWARD;
  ev.data_len = sizeof(PhPointerCtrlEvent_t);

  PhEventEmit(&ev, NULL, &PtrCtrlEvent);

  PtRealizeWidget(TouchRegion);

  PtTimerArm( ABW_timer, 500 );
}

int base_setup( PtWidget_t *widget, ApInfo_t *apinfo, 
                PtCallbackInfo_t *cbinfo )
{
  PtArg_t args[1];

  signal( SIGTERM, base_signal );
  pointer_raw();

  return( Pt_CONTINUE );
}

