/*
 * Copyright 2001, QNX Software Systems Ltd. All Rights Reserved
 *
 * This source code has been published by QNX Software Systems Ltd. (QSSL).
 * However, any use, reproduction, modification, distribution or transfer of
 * this software, or any software which includes or is based upon any of this
 * code, is only permitted under the terms of the QNX Open Community License
 * version 1.0 (see licensing.qnx.com for details) or as otherwise expressly
 * authorized by a written license agreement from QSSL. For more information,
 * please email licensing@qnx.com.
 */


#ifndef _ERRDBG_H_INCLUDED
#define _ERRDBG_H_INCLUDED

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


/* 
 * examples of the DEBUG macro in action
 *
 * DBG(printf("This val should be: %d\n", val););
 *
 * DBG(assert(x == someval););
 *
 * DBG(if (x == someval) 
         print("Oops");
       else 
         x = newval;
   );
 *
 *
 */

#ifdef DEBUG
#define DBG(code_fragment) do { code_fragment } while (0)
#else
#define DBG(code_fragment)  // debug: do nothing
#endif

#define VBOSE(a) if(verbose) printf("%s\n",(a))

#define err_abort(code, text) do { \
		     fprintf (stderr, "%s at \"%s\":%d %s\n", \
			      text, __FILE__, __LINE__, strerror(code)); \
		     abort(); \
		     } while (0)

#define errno_abort(text) do { \
		     fprintf (stderr, "%s at \"%s\":%d: %s\n", \
			      text, __FILE__, __LINE__, strerror(errno)); \
		     abort(); \
		     } while (0)


#define errno_exit(text) do { \
		     fprintf (stderr, "%s at \"%s\":%d: %s\n", \
			      text, __FILE__, __LINE__, strerror(errno)); \
		     exit(EXIT_FAILURE); \
		     } while (0)


#define errno_print(text) do { \
		     fprintf (stderr, "%s at \"%s\":%d: %s\n", \
			      text, __FILE__, __LINE__, strerror(errno)); \
		     } while (0)




#endif
